package net.minecraft.client.resources;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Set;

import com.google.common.collect.ImmutableSet;

import net.minecraft.client.renderer.texture.TextureUtil;
import net.minecraft.client.resources.data.IMetadataSection;
import net.minecraft.client.resources.data.IMetadataSerializer;
import net.minecraft.src.ReflectorForge;
import net.minecraft.util.ResourceLocation;

public class DefaultResourcePack implements IResourcePack {
	public static final Set defaultResourceDomains = ImmutableSet.of("minecraft", "realms");
	private final Map mapAssets;
	private static final String __OBFID = "CL_00001073";

	public DefaultResourcePack(Map mapAssetsIn) {
		this.mapAssets = mapAssetsIn;
	}

	public InputStream getInputStream(ResourceLocation location) throws IOException {
		InputStream inputstream = this.getResourceStream(location);

		if (inputstream != null) {
			return inputstream;
		} else {
			InputStream inputstream1 = this.getInputStreamAssets(location);

			if (inputstream1 != null) {
				return inputstream1;
			} else {
				throw new FileNotFoundException(location.getResourcePath());
			}
		}
	}

	public InputStream getInputStreamAssets(ResourceLocation location) throws IOException, FileNotFoundException {
		File file1 = (File) this.mapAssets.get(location.toString());
		return file1 != null && file1.isFile() ? new FileInputStream(file1) : null;
	}

	private InputStream getResourceStream(ResourceLocation location) {
		String s = "/assets/" + location.getResourceDomain() + "/" + location.getResourcePath();
		InputStream inputstream = ReflectorForge.getOptiFineResourceStream(s);
		return inputstream != null ? inputstream
				: DefaultResourcePack.class.getResourceAsStream(
						"/assets/" + location.getResourceDomain() + "/" + location.getResourcePath());
	}

	public boolean resourceExists(ResourceLocation location) {
		return this.getResourceStream(location) != null || this.mapAssets.containsKey(location.toString());
	}

	public Set getResourceDomains() {
		return defaultResourceDomains;
	}

	public IMetadataSection getPackMetadata(IMetadataSerializer p_135058_1_, String p_135058_2_) throws IOException {
		try {
			FileInputStream fileinputstream = new FileInputStream((File) this.mapAssets.get("pack.mcmeta"));
			return AbstractResourcePack.readMetadata(p_135058_1_, fileinputstream, p_135058_2_);
		} catch (RuntimeException var4) {
			return null;
		} catch (FileNotFoundException var5) {
			return null;
		}
	}

	public BufferedImage getPackImage() throws IOException {
		return TextureUtil.readBufferedImage(DefaultResourcePack.class
				.getResourceAsStream("/" + (new ResourceLocation("pack.png")).getResourcePath()));
	}

	public String getPackName() {
		return "Default";
	}
}
