package net.minecraft.network.play.server;

import java.io.IOException;

import net.minecraft.network.AbstractPacket;
import net.minecraft.network.PacketBuffer;
import net.minecraft.network.play.INetHandlerPlayClient;
import net.minecraft.util.BlockPos;

public class S25PacketBlockBreakAnim extends AbstractPacket<INetHandlerPlayClient> {
	private int breakerId;
	private BlockPos position;
	private int progress;

	public S25PacketBlockBreakAnim() {
	}

	public S25PacketBlockBreakAnim(int breakerId, BlockPos pos, int progress) {
		this.breakerId = breakerId;
		this.position = pos;
		this.progress = progress;
	}

	/**
	 * Reads the raw packet data from the data stream.
	 */
	public void readPacketData(PacketBuffer buf) throws IOException {
		this.breakerId = buf.readVarIntFromBuffer();
		this.position = buf.readBlockPos();
		this.progress = buf.readUnsignedByte();
	}

	/**
	 * Writes the raw packet data to the data stream.
	 */
	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeVarIntToBuffer(this.breakerId);
		buf.writeBlockPos(this.position);
		buf.writeByte(this.progress);
	}

	/**
	 * Passes this Packet on to the NetHandler for processing.
	 */
	public void processPacket(INetHandlerPlayClient handler) {
		super.processPacket(handler);
		if (cancelled) {
			return;
		}
		handler.handleBlockBreakAnim(this);
	}

	public int getBreakerId() {
		return this.breakerId;
	}

	public BlockPos getPosition() {
		return this.position;
	}

	public int getProgress() {
		return this.progress;
	}
}
