package shadersmod.client;

import java.nio.FloatBuffer;
import java.util.Arrays;

public class SMath {
	static void multiplyMat4xMat4(float[] matOut, float[] matA, float[] matB) {
		for (int i = 0; i < 4; ++i) {
			for (int j = 0; j < 4; ++j) {
				matOut[4 * i + j] = matA[4 * i + 0] * matB[0 + j] + matA[4 * i + 1] * matB[4 + j]
						+ matA[4 * i + 2] * matB[8 + j] + matA[4 * i + 3] * matB[12 + j];
			}
		}
	}

	static void multiplyMat4xVec4(float[] vecOut, float[] matA, float[] vecB) {
		vecOut[0] = matA[0] * vecB[0] + matA[4] * vecB[1] + matA[8] * vecB[2] + matA[12] * vecB[3];
		vecOut[1] = matA[1] * vecB[0] + matA[5] * vecB[1] + matA[9] * vecB[2] + matA[13] * vecB[3];
		vecOut[2] = matA[2] * vecB[0] + matA[6] * vecB[1] + matA[10] * vecB[2] + matA[14] * vecB[3];
		vecOut[3] = matA[3] * vecB[0] + matA[7] * vecB[1] + matA[11] * vecB[2] + matA[15] * vecB[3];
	}

	static void invertMat4(float[] matOut, float[] m) {
		matOut[0] = m[5] * m[10] * m[15] - m[5] * m[11] * m[14] - m[9] * m[6] * m[15] + m[9] * m[7] * m[14]
				+ m[13] * m[6] * m[11] - m[13] * m[7] * m[10];
		matOut[1] = -m[1] * m[10] * m[15] + m[1] * m[11] * m[14] + m[9] * m[2] * m[15] - m[9] * m[3] * m[14]
				- m[13] * m[2] * m[11] + m[13] * m[3] * m[10];
		matOut[2] = m[1] * m[6] * m[15] - m[1] * m[7] * m[14] - m[5] * m[2] * m[15] + m[5] * m[3] * m[14]
				+ m[13] * m[2] * m[7] - m[13] * m[3] * m[6];
		matOut[3] = -m[1] * m[6] * m[11] + m[1] * m[7] * m[10] + m[5] * m[2] * m[11] - m[5] * m[3] * m[10]
				- m[9] * m[2] * m[7] + m[9] * m[3] * m[6];
		matOut[4] = -m[4] * m[10] * m[15] + m[4] * m[11] * m[14] + m[8] * m[6] * m[15] - m[8] * m[7] * m[14]
				- m[12] * m[6] * m[11] + m[12] * m[7] * m[10];
		matOut[5] = m[0] * m[10] * m[15] - m[0] * m[11] * m[14] - m[8] * m[2] * m[15] + m[8] * m[3] * m[14]
				+ m[12] * m[2] * m[11] - m[12] * m[3] * m[10];
		matOut[6] = -m[0] * m[6] * m[15] + m[0] * m[7] * m[14] + m[4] * m[2] * m[15] - m[4] * m[3] * m[14]
				- m[12] * m[2] * m[7] + m[12] * m[3] * m[6];
		matOut[7] = m[0] * m[6] * m[11] - m[0] * m[7] * m[10] - m[4] * m[2] * m[11] + m[4] * m[3] * m[10]
				+ m[8] * m[2] * m[7] - m[8] * m[3] * m[6];
		matOut[8] = m[4] * m[9] * m[15] - m[4] * m[11] * m[13] - m[8] * m[5] * m[15] + m[8] * m[7] * m[13]
				+ m[12] * m[5] * m[11] - m[12] * m[7] * m[9];
		matOut[9] = -m[0] * m[9] * m[15] + m[0] * m[11] * m[13] + m[8] * m[1] * m[15] - m[8] * m[3] * m[13]
				- m[12] * m[1] * m[11] + m[12] * m[3] * m[9];
		matOut[10] = m[0] * m[5] * m[15] - m[0] * m[7] * m[13] - m[4] * m[1] * m[15] + m[4] * m[3] * m[13]
				+ m[12] * m[1] * m[7] - m[12] * m[3] * m[5];
		matOut[11] = -m[0] * m[5] * m[11] + m[0] * m[7] * m[9] + m[4] * m[1] * m[11] - m[4] * m[3] * m[9]
				- m[8] * m[1] * m[7] + m[8] * m[3] * m[5];
		matOut[12] = -m[4] * m[9] * m[14] + m[4] * m[10] * m[13] + m[8] * m[5] * m[14] - m[8] * m[6] * m[13]
				- m[12] * m[5] * m[10] + m[12] * m[6] * m[9];
		matOut[13] = m[0] * m[9] * m[14] - m[0] * m[10] * m[13] - m[8] * m[1] * m[14] + m[8] * m[2] * m[13]
				+ m[12] * m[1] * m[10] - m[12] * m[2] * m[9];
		matOut[14] = -m[0] * m[5] * m[14] + m[0] * m[6] * m[13] + m[4] * m[1] * m[14] - m[4] * m[2] * m[13]
				- m[12] * m[1] * m[6] + m[12] * m[2] * m[5];
		matOut[15] = m[0] * m[5] * m[10] - m[0] * m[6] * m[9] - m[4] * m[1] * m[10] + m[4] * m[2] * m[9]
				+ m[8] * m[1] * m[6] - m[8] * m[2] * m[5];
		float f = m[0] * matOut[0] + m[1] * matOut[4] + m[2] * matOut[8] + m[3] * matOut[12];

		if ((double) f != 0.0D) {
			for (int i = 0; i < 16; ++i) {
				matOut[i] /= f;
			}
		} else {
			Arrays.fill(matOut, 0.0F);
		}
	}

	static void invertMat4FBFA(FloatBuffer fbInvOut, FloatBuffer fbMatIn, float[] faInv, float[] faMat) {
		fbMatIn.get(faMat);
		invertMat4(faInv, faMat);
		fbInvOut.put(faInv);
	}
}
